#ifndef __EVALUATION_H__
#define __EVALUATION_H__

#include "cv.h"
#include "trackingobject.h"

#include <vector>
#include <libxml/xmlreader.h>


enum GROUND_TRUTH_TYPE { CAVIAR_XML, MUSCLE_TXT };

/* The class PerformanceEvaluator is used for evaluting tracking performance
 *
 * it needs (usually manually annotated) ground truth data
 */
class PerformanceEvaluator
{
	public:
		PerformanceEvaluator();
		virtual ~PerformanceEvaluator(){}

		// for each frame in the video sequence call this function
		void addFrameInformation(TrackingObject *object);

		// all these functions return a percentage value
		int getID() { return id; }
		float overlappingBoundingBoxes() { if(!frames_evaluated) return 1.f; else return overlapping_bboxes/frames_evaluated;};
		float overlappingSegmentation()  { return 1;}; // will only work for synthetic data
		float successfullyTracked()      { if(!frames_evaluated) return 1.f; else return (float)frames_successfully_tracked/frames_evaluated;};


		CvPoint curr_gt_center;
		CvRect  curr_gt_bbox;
		CvPoint point_of_id; // set this point when you know the coordinate of a point of the tracked object

	protected:
		virtual bool readNextFrame() = 0;
		int id;
		int frames_evaluated;
		int frames_successfully_tracked;
		double overlapping_bboxes;
};


class MusclePerformanceEvaluator : public PerformanceEvaluator
{
	public:
		MusclePerformanceEvaluator(const char* ground_truth_file, int skip_frames);
		~MusclePerformanceEvaluator();

	protected:
		FILE* file;
		bool readNextFrame();
};

class CaviarPerformanceEvaluator : public PerformanceEvaluator
{
	public:
		CaviarPerformanceEvaluator(const char* ground_truth_file, int skip_frames);
		~CaviarPerformanceEvaluator();

	protected:
	    xmlTextReaderPtr reader;
		bool readNextFrame();
};

#endif /* __EVALUATION_H__ */
