#ifndef __FLOW_H__
#define __FLOW_H__

#include "cv.h"
#include "config.h" // for MAX_FEATURE_POINTS, etc.
#include "trackingobject.h" // for MAX_FEATURE_POINTS, etc.
#include <vector>

/* The class OpticalFlow is used to find certain
 * feature points from one image in the next image of a video sequence
 *
 * This implementation uses the Lucas & Kanade method
 */

// needed to access global variable
extern std::vector<TrackingObject*> *tracking_objects;

class OpticalFlow
{
	public:
		OpticalFlow(const IplImage *first_image, const CvPoint2D32f* initial_features = NULL, const int initial_feature_count = 0);
		virtual ~OpticalFlow();

		// find feature points from the first image in the second image
		// returns number of found feature points
		virtual int updateFlow(const IplImage* next_image, CvPoint2D32f** features, char** status, int* lost_feature_count);

		void setCurrentFeaturePoints(const CvPoint2D32f* points, const char* status, const int num_points);
		
		CvPoint2D32f curr_features[MAX_FEATURE_POINTS];
		CvPoint2D32f guess_features[MAX_FEATURE_POINTS];
		char curr_status[MAX_FEATURE_POINTS];
		int num_features;

	private:
		IplImage *prev_pyramid, *curr_pyramid;
		float error[MAX_FEATURE_POINTS];

		void guess(CvPoint2D32f* features, TrackingObject *prev);
		int correct(CvPoint2D32f* features, TrackingObject *prev);
};

#endif /* __FLOW_H__ */
