#ifndef  __TRACKINGOBJECT_H__
#define  __TRACKINGOBJECT_H__

#include "cv.h"
#include "segmentation.h"
#include "config.h"

#include <set>

class TrackingObject
{
	public:
		TrackingObject(IplImage *image, CvPoint2D32f* points = NULL, char* status = NULL, int num = 0);
		~TrackingObject();
		TrackingObject(const TrackingObject &pts);

		CvPoint2D32f getCenter();
		CvPoint2D32f getDirection(TrackingObject* prev = NULL);
		void finalize(TrackingObject* prev);

		// segmentation related functions/variables //////////////////////
		void addSegment(Component* comp);
		CvRect getBoundingBox();         // returns the bounding box for ALL segments in this object
		std::set<Component*> *active_segments;  // the segments which make up the whole object

		// flow related functions/variables //////////////////////////////
		void addFeaturePoints(); // this version calculated feature points automatically from the segments
		void addFeaturePoints(CvPoint2D32f* points, char* status, int num);
		CvPoint2D32f features[MAX_FEATURE_POINTS];
		char status[MAX_FEATURE_POINTS];
		float error[MAX_FEATURE_POINTS];
		int num_features, num_found_features, num_predicted_features, num_lost_features;

		IplImage *gray_image; // we store a copy of the gray image

	private:
		bool finalized;

		CvPoint2D32f center;
		CvPoint2D32f direction;
		CvRect bbox;
};

#endif  /*__TRACKINGOBJECT_H__*/
